from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_api.filter.auth_user_select_criteria import AuthUserSelectCriteria
from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.auth_user_repository_abc import AuthUserRepositoryABC
from bot_data.filtered_result import FilteredResult
from bot_data.model.auth_role_enum import AuthRoleEnum
from bot_data.model.auth_user import AuthUser


class AuthUserRepositoryService(AuthUserRepositoryABC):

    def __init__(self, logger: DatabaseLogger, db_context: DatabaseContextABC):
        self._logger = logger
        self._context = db_context

        AuthUserRepositoryABC.__init__(self)

    @staticmethod
    def _get_value_from_result(value: any) -> Optional[any]:
        if isinstance(value, str) and 'NULL' in value:
            return None

        return value

    def _user_from_result(self, result: tuple) -> AuthUser:
        return AuthUser(
            self._get_value_from_result(result[1]),
            self._get_value_from_result(result[2]),
            self._get_value_from_result(result[3]),
            self._get_value_from_result(result[4]),
            self._get_value_from_result(result[5]),
            self._get_value_from_result(result[6]),
            self._get_value_from_result(result[7]),
            self._get_value_from_result(result[8]),
            AuthRoleEnum(self._get_value_from_result(result[9])),
            self._get_value_from_result(result[10]),
            id=self._get_value_from_result(result[0])
        )

    def get_all_auth_users(self) -> List[AuthUser]:
        users = List(AuthUser)
        self._logger.trace(__name__, f'Send SQL command: {AuthUser.get_select_all_string()}')
        results = self._context.select(AuthUser.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f'Get auth user with id {result[0]}')
            users.append(self._user_from_result(result))

        return users

    def get_filtered_auth_users(self, criteria: AuthUserSelectCriteria) -> FilteredResult:
        users = self.get_all_auth_users()
        self._logger.trace(__name__, f'Send SQL command: {AuthUser.get_select_all_string()}')
        query = users

        if criteria.first_name is not None and criteria.first_name != '':
            query = query.where(lambda x: criteria.first_name in x.first_name or x.first_name == criteria.first_name)

        if criteria.last_name is not None and criteria.last_name != '':
            query = query.where(lambda x: criteria.last_name in x.last_name or x.last_name == criteria.last_name)

        if criteria.email is not None and criteria.email != '':
            query = query.where(lambda x: criteria.email in x.email or x.email == criteria.email)

        if criteria.auth_role is not None:
            query = query.where(lambda x: x.auth_role == AuthRoleEnum(criteria.auth_role))

        # sort
        if criteria.sort_column is not None and criteria.sort_column != '' and criteria.sort_direction is not None and criteria.sort_direction:
            crit_sort_direction = criteria.sort_direction.lower()
            if crit_sort_direction == "desc" or crit_sort_direction == "descending":
                query = query.order_by_descending(lambda x: getattr(x, criteria.sort_column))
            else:
                query = query.order_by(lambda x: getattr(x, criteria.sort_column))

        result = FilteredResult()
        result.total_count = query.count()
        skip = criteria.page_size * criteria.page_index
        result.result = query.skip(skip).take(criteria.page_size)

        return result

    def get_auth_user_by_email(self, email: str) -> AuthUser:
        self._logger.trace(__name__, f'Send SQL command: {AuthUser.get_select_by_email_string(email)}')
        result = self._context.select(AuthUser.get_select_by_email_string(email))[0]
        return self._user_from_result(result)

    def find_auth_user_by_email(self, email: str) -> Optional[AuthUser]:
        self._logger.trace(__name__, f'Send SQL command: {AuthUser.get_select_by_email_string(email)}')
        result = self._context.select(AuthUser.get_select_by_email_string(email))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return self._user_from_result(result)

    def find_auth_user_by_confirmation_id(self, id: str) -> Optional[AuthUser]:
        self._logger.trace(__name__, f'Send SQL command: {AuthUser.get_select_by_confirmation_id_string(id)}')
        result = self._context.select(AuthUser.get_select_by_confirmation_id_string(id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return self._user_from_result(result)

    def find_auth_user_by_forgot_password_id(self, id: str) -> Optional[AuthUser]:
        self._logger.trace(__name__, f'Send SQL command: {AuthUser.get_select_by_forgot_password_id_string(id)}')
        result = self._context.select(AuthUser.get_select_by_forgot_password_id_string(id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return self._user_from_result(result)

    def add_auth_user(self, user: AuthUser):
        self._logger.trace(__name__, f'Send SQL command: {user.insert_string}')
        self._context.cursor.execute(user.insert_string)

    def update_auth_user(self, user: AuthUser):
        self._logger.trace(__name__, f'Send SQL command: {user.udpate_string}')
        self._context.cursor.execute(user.udpate_string)

    def delete_auth_user(self, user: AuthUser):
        self._logger.trace(__name__, f'Send SQL command: {user.delete_string}')
        self._context.cursor.execute(user.delete_string)
